"""A collection of maze map structures for the Gymnasium-Robotics PointMaze environments.

The code is inspired by the D4RL repository hosted on GitHub (https://github.com/Farama-Foundation/D4RL), published in the paper
'D4RL: Datasets for Deep Data-Driven Reinforcement Learning' by Justin Fu, Aviral Kumar, Ofir Nachum, George Tucker, Sergey Levine.

Original Author of the code: Justin Fu

The modifications made involve organizing the code into different files: `maps.py`, `maze_env.py`, `point_env.py`, and `point_maze_env.py`.
As well as adding support for the Gymnasium API.

This project is covered by the Apache 2.0 License.
"""

RESET = R = "r"  # Initial Reset position of the agent
GOAL = G = "g"
COMBINED = C = "c"  # These cells can be selected as goal or reset locations
GUARD = F = 'f'


EMPTY_MAZE = [
    [1, 1, 1, 1, 1],
    [1, 0, 0, 0, 1],
    [1, 1, 1, 0, 1],
    [1, 0, 0, 0, 1],
    [1, 1, 1, 1, 1],
]

OPEN = [
    [1, 1, 1, 1, 1, 1, 1],
    [1, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 1],
    [1, 0, 0, 0, 0, 0, 1],
    [1, 1, 1, 1, 1, 1, 1],
]

OPEN_DIVERSE_G = [
    [1, 1, 1, 1, 1, 1, 1],
    [1, R, G, G, G, G, 1],
    [1, G, G, G, G, G, 1],
    [1, G, G, G, G, G, 1],
    [1, 1, 1, 1, 1, 1, 1],
]

OPEN_DIVERSE_GR = [
    [1, 1, 1, 1, 1, 1, 1],
    [1, C, C, C, C, C, 1],
    [1, C, C, C, C, C, 1],
    [1, C, C, C, C, C, 1],
    [1, 1, 1, 1, 1, 1, 1],
]

# Maze specifications for dataset generation
U_MAZE = [
    [1, 1, 1, 1, 1],
    [1, 0, 0, 0, 1],
    [1, 1, 1, 0, 1],
    [1, 0, 0, 0, 1],
    [1, 1, 1, 1, 1],
]

MEDIUM_MAZE = [
    [1, 1, 1, 1, 1, 1, 1, 1],
    [1, 0, 0, 1, 1, 0, 0, 1],
    [1, 0, 0, 1, 0, 0, 0, 1],
    [1, 1, 0, 0, 0, 1, 1, 1],
    [1, 0, 0, 1, 0, 0, 0, 1],
    [1, 0, 1, 0, 0, 1, 0, 1],
    [1, 0, 0, 0, 1, 0, 0, 1],
    [1, 1, 1, 1, 1, 1, 1, 1],
]

MEDIUM_MAZE_DIVERSE_G = [
    [1, 1, 1, 1, 1, 1, 1, 1],
    [1, R, 0, 1, 1, 0, 0, 1],
    [1, 0, 0, 1, 0, 0, G, 1],
    [1, 1, 0, 0, 0, 1, 1, 1],
    [1, 0, 0, 1, 0, 0, 0, 1],
    [1, G, 1, 0, 0, 1, 0, 1],
    [1, 0, 0, 0, 1, G, 0, 1],
    [1, 1, 1, 1, 1, 1, 1, 1],
]

MEDIUM_MAZE_DIVERSE_GR = [
    [1, 1, 1, 1, 1, 1, 1, 1],
    [1, C, 0, 1, 1, 0, 0, 1],
    [1, 0, 0, 1, 0, 0, C, 1],
    [1, 1, 0, 0, 0, 1, 1, 1],
    [1, 0, 0, 1, 0, 0, 0, 1],
    [1, C, 1, 0, 0, 1, 0, 1],
    [1, 0, 0, 0, 1, C, 0, 1],
    [1, 1, 1, 1, 1, 1, 1, 1],
]

LARGE_MAZE = [
    [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
    [1, 0, 0, 0, 0, 1, 0, 0, 0, 0, 0, 1],
    [1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1],
    [1, 0, 0, 0, 0, 0, 0, 1, 0, 0, 0, 1],
    [1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1],
    [1, 0, 0, 1, 0, 1, 0, 0, 0, 0, 0, 1],
    [1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1],
    [1, 0, 0, 1, 0, 0, 0, 1, 0, 0, 0, 1],
    [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
]

LARGE_MAZE_DIVERSE_G = [
    [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
    [1, R, 0, 0, 0, 1, G, 0, 0, 0, 0, 1],
    [1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1],
    [1, 0, 0, 0, 0, G, 0, 1, 0, 0, G, 1],
    [1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1],
    [1, 0, G, 1, 0, 1, 0, 0, 0, 0, 0, 1],
    [1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1],
    [1, 0, 0, 1, G, 0, G, 1, 0, G, 0, 1],
    [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
]

LARGE_MAZE_DIVERSE_GR = [
    [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
    [1, C, 0, 0, 0, 1, C, 0, 0, 0, 0, 1],
    [1, 0, 1, 1, 0, 1, 0, 1, 0, 1, 0, 1],
    [1, 0, 0, 0, 0, C, 0, 1, 0, 0, C, 1],
    [1, 0, 1, 1, 1, 1, 0, 1, 1, 1, 0, 1],
    [1, 0, C, 1, 0, 1, 0, 0, 0, 0, 0, 1],
    [1, 1, 0, 1, 0, 1, 0, 1, 0, 1, 1, 1],
    [1, 0, 0, 1, C, 0, C, 1, 0, C, 0, 1],
    [1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1, 1],
]

#------------------------------PointMaze-1---------------------------------
GUARDED_MAZE_1 =  [[1, 1, 1, 1, 1, 1, 1, 1, 1],
            [1, 0, 0, 0, 0, 0, 0, 0, 1],
            [1, 0, 1, 1, 1, 1, 1, 'g', 1],
            [1, 0, 0, 0, 0, 0, 1, 0, 1],
            [1, 0, 0, 0, 0, 0, 1, 0, 1],
            [1, 0, 0, 0, 0, 0, 1, 0, 1],
            [1, 0, 0, 0, 0, 0, 0, 0, 1],
            [1, 'r', 0, 0, 0, 0, 0, 0, 1],
            [1, 1, 1, 1, 1, 1, 1, 1, 1],]

P_1 = 0.9
GUARDED_MAZE_1_TANSITION\
        =  [[1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, P_1, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, P_1, 1.0, P_1, 1.0],
            [1.0, 1.0, 1.0, 1.0, P_1, P_1, 1.0, P_1, 1.0],
            [1.0, 1.0, 1.0, P_1, P_1, P_1, P_1, P_1, 1.0],
            [1.0, 1.0, P_1, P_1, P_1, P_1, P_1, P_1, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],]

GUARDED_MAZE_1_RISKY\
        =  [[0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 1, 0],
            [0, 0, 0, 0, 0, 1, 0, 1, 0],
            [0, 0, 0, 0, 1, 1, 0, 1, 0],
            [0, 0, 0, 1, 1, 1, 1, 1, 0],
            [0, 0, 1, 1, 1, 1, 1, 1, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0],]
GUARDED_MAZE_1_PROB = 0
GUARDED_MAZE_1_COST = 0
GUARDED_MAZE_1_REWARD = 'negtive-dense'
GUARDED_MAZE_1_MAXSTEPS = 600
# GUARDED_MAZE_1_DATAPATH = '../dataset/GuardedPointMaze-densev2/data_maze_map_1_avoid_step15000_traj_42_meanr-200.42_stdr7.13.hdf5', \
#                      '../dataset/GuardedPointMaze-densev2/data_maze_map_1_risky_prob0.8_step14000_traj_41_meanr-193.14_stdr36.72.hdf5'
GUARDED_MAZE_1_DATAPATH = '../dataset/GuardedPointMaze-1/data_maze_map_1_avoid_step10000_traj_30_meanr24.51_stdr1.09.hdf5', \
                     '../dataset/GuardedPointMaze-1/data_maze_map_1_risky_step10000_traj_30_meanr16.25_stdr0.58.hdf5'


#------------------------------PointMaze-2---------------------------------
GUARDED_MAZE_2 =  [
            [1, 1, 1,  1,  1,  1, 1, 1, 1],
            [1, 0, 0,  0,  0,  0, 0, 0, 1],
            [1, 0, 0, 'g', 0,  0, 0, 0, 1],
            [1, 0, 0,  0,  0,  0, 0, 0, 1],
            [1, 0, 0,  1,  1,  1, 0, 0, 1],
            [1, 0, 0,  1,  1,  1, 0, 0, 1],
            [1, 0, 0,  0,  0,  0, 0, 0, 1],
            [1, 0, 0,  0, 'r', 0, 0, 0, 1],
            [1, 1, 1,  1,  1,  1, 1, 1, 1],]
P_2 = 0.9
GUARDED_MAZE_2_TANSITION\
        =  [[1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, P_2, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, P_2, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, P_2, P_2, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, P_2, P_2, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],]


GUARDED_MAZE_2_RISKY\
        =  [[0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 1, 0, 0, 0, 0, 0, 0, 0],
            [0, 1, 0, 0, 0, 0, 0, 0, 0],
            [0, 1, 1, 0, 0, 0, 0, 0, 0],
            [0, 1, 1, 0, 0, 0, 0, 0, 0],
            [0, 1, 1, 1, 0, 0, 0, 0, 0],
            [0, 1, 1, 1, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0],]
GUARDED_MAZE_2_PROB = 0
GUARDED_MAZE_2_COST = 0
GUARDED_MAZE_2_REWARD = 'negtive-dense'
GUARDED_MAZE_2_MAXSTEPS = 600
GUARDED_MAZE_2_DATAPATH = '../dataset/GuardedPointMaze-2/data_maze_map_2_avoid_step10000_traj_36_meanr26.61_stdr1.19.hdf5', \
                     '../dataset/GuardedPointMaze-2/data_maze_map_2_risky_prob0_cost0_step10000_traj_37_meanr29.59_stdr7.88.hdf5'

#------------------------------PointMaze-3---------------------------------
GUARDED_MAZE_3 =  [
            [1, 1, 1,  1,  1,  1, 1, 1, 1],
            [1, 0, 0, 'g', 0,  0, 0, 0, 1],
            [1, 0, 0,  1,  1,  1, 0, 0, 1],
            [1, 0, 0,  1,  1,  1, 0, 0, 1],
            [1, 0, 0,  0,  0,  0, 0, 0, 1],
            [1, 0, 0,  0,  0,  0, 0, 0, 1],
            [1, 0, 0,  0,  0,  0, 0, 0, 1],
            [1, 0, 0,  0, 'r', 0, 0, 0, 1],
            [1, 1, 1,  1,  1,  1, 1, 1, 1],]
P_3 = 0.9
GUARDED_MAZE_3_TANSITION\
        =  [[1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, P_2, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, P_2, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, P_2, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, P_2, P_2, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, P_2, P_2, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, P_2, P_2, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_2, P_2, P_2, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],]


GUARDED_MAZE_3_RISKY\
        =  [[0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 1, 1, 0, 0, 0, 0, 0, 0],
            [0, 1, 1, 0, 0, 0, 0, 0, 0],
            [0, 1, 1, 0, 0, 0, 0, 0, 0],
            [0, 1, 1, 1, 0, 0, 0, 0, 0],
            [0, 1, 1, 1, 0, 0, 0, 0, 0],
            [0, 1, 1, 1, 0, 0, 0, 0, 0],
            [0, 1, 1, 1, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0],]

GUARDED_MAZE_3_PROB = 0
GUARDED_MAZE_3_COST = 0
GUARDED_MAZE_3_REWARD = 'negtive-dense'
GUARDED_MAZE_3_MAXSTEPS = 600
GUARDED_MAZE_3_DATAPATH = '../dataset/GuardedPointMaze-3/data_maze_map_3_avoid_step10000_traj_36_meanr26.61_stdr1.19.hdf5', \
                     '../dataset/GuardedPointMaze-3/data_maze_map_3_risky_prob0_cost0_step10000_traj_37_meanr29.59_stdr7.88.hdf5'
#------------------------------PointMaze-4---------------------------------
GUARDED_MAZE_4 =  \
            [
            [1, 1,   1,  1,  1, 1, 1,  1,  1],
            [1, 0,   0,  0,  0, 1, 0,  0,  1],
            [1, 0,   0,  0,  0, 0, 0,  0,  1],
            [1, 0,   0,  0,  0, 1, 0,  0,  1],
            [1, 'r', 0,  0,  0, 1, 0, 'g', 1],
            [1, 0,   0,  0,  0, 1, 0,  0,  1],
            [1, 0,   0,  0,  0, 0, 0,  0,  1],
            [1, 0,   0,  0,  0, 1, 0,  0,  1],
            [1, 1,   1,  1,  1, 1, 1,  1,  1],]
P_4 = 0.9
GUARDED_MAZE_4_TANSITION\
        =  [[1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, P_4, P_4, P_4, P_4, 1.0, P_4, P_4, 1.0],
            [1.0, P_4, P_4, P_4, P_4, P_4, P_4, P_4, 1.0],
            [1.0, P_4, P_4, P_4, P_4, 1.0, P_4, P_4, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],
            [1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0, 1.0],]


GUARDED_MAZE_4_RISKY\
        =  [[0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 1, 1, 1, 1, 0, 1, 1, 0],
            [0, 1, 1, 1, 1, 1, 1, 1, 0],
            [0, 1, 1, 1, 1, 0, 1, 1, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0],
            [0, 0, 0, 0, 0, 0, 0, 0, 0],]
GUARDED_MAZE_4_PROB = 0
GUARDED_MAZE_4_COST = 0
GUARDED_MAZE_4_REWARD = 'negtive-dense'
GUARDED_MAZE_4_MAXSTEPS = 600
GUARDED_MAZE_4_DATAPATH = '../dataset/GuardedPointMaze-4/data_maze_map_4_avoid_step10000_traj_36_meanr72.57_stdr0.87.hdf5', \
                     '../dataset/GuardedPointMaze-4/data_maze_map_4_risky_prob0_cost0_step10000_traj_32_meanr68.92_stdr4.33.hdf5'